-----------------------------------------------------------------------------------------------
-- Client Lua Script for BetterBotBar
-- Copyright (c) NCsoft. All rights reserved
-----------------------------------------------------------------------------------------------
 
require "Window"
require "Apollo"
 
-----------------------------------------------------------------------------------------------
-- BetterBotBar Module Definition
-----------------------------------------------------------------------------------------------
local BetterBotBar = {} 
 
-----------------------------------------------------------------------------------------------
-- Constants
-----------------------------------------------------------------------------------------------
-- e.g. local kiExampleVariableMax = 999

local defaultSettings = {
	barPosition = {-109, -255, 97, -157},
	bNeverShowBotBar = false,
	bPetBtnPressed = false,
	bLockBar = false
}

local userSettings = {
	barPosition = defaultSettings.barPosition,
	bNeverShowBotBar = defaultSettings.bNeverShowBotBar,
	bPetBtnPressed = defaultSettings.bPetBtnPressed,
	bLockBar = defaultSettings.bLockBar
}
-----------------------------------------------------------------------------------------------
-- Initialization
-----------------------------------------------------------------------------------------------
function BetterBotBar:new(o)
    o = o or {}
    setmetatable(o, self)
    self.__index = self 

    -- initialize variables here

	
    return o
end

function BetterBotBar:Init()
	local bHasConfigureFunction = false
	local strConfigureButtonText = ""
	local tDependencies = {
		"ClassResources"
	}
    Apollo.RegisterAddon(self, bHasConfigureFunction, strConfigureButtonText, tDependencies)
end

-----------------------------------------------------------------------------------------------
-- Save and Restore
-----------------------------------------------------------------------------------------------
function BetterBotBar:OnSave(eType)
	if eType ~= GameLib.CodeEnumAddonSaveLevel.Character then
		return
	end
		
	self:SaveBarPosition(userSettings.barPosition)
	
	return userSettings
	
end

function BetterBotBar:OnRestore(eType, tSave)
	if eType ~= GameLib.CodeEnumAddonSaveLevel.Character then
		return
	end
		
	if tSave then
		for k,v in pairs(tSave) do
			userSettings[k] = v
		end
	end
	
	if tSave.barPosition then
		for k,v in pairs(tSave.barPosition) do
			userSettings.barPosition[k] = v
		end
	end
		
	self.onRestoreCalled = true
	self:UpdateSettings()
	
end

-----------------------------------------------------------------------------------------------
-- BetterBotBar OnLoad
-----------------------------------------------------------------------------------------------
function BetterBotBar:OnLoad()
	
	self:InitializeHooks()
	
	-- Initialize fields
	
	self.onRestoreCalled = false
	self.onXmlDocLoadedCalled = false
	
	isFirstBotCall = true
	
    -- load our form file
	self.xmlDoc = XmlDoc.CreateFromFile("BetterBotBar.xml")
	self.xmlDoc:RegisterCallback("OnDocLoaded", self)
	
end

function BetterBotBar:InitializeHooks()
	
	crbClassResources = Apollo.GetAddon("ClassResources")
	if crbClassResources ~= nil then
		crbClassResources.OnShowActionBarShortcut = self.OnShowActionBarShortcut
		crbClassResources.OnPetBtn = self.OnPetBtn
	end
	
end


-----------------------------------------------------------------------------------------------
-- BetterBotBar OnDocLoaded
-----------------------------------------------------------------------------------------------
function BetterBotBar:OnDocLoaded()
	if self.xmlDoc ~= nil and self.xmlDoc:IsLoaded() then
	    self.wndSettings = Apollo.LoadForm(self.xmlDoc, "BbbSettings", nil, self)
		if self.wndSettings == nil then
			Apollo.AddAddonErrorText(self, "Could not load the settings window for some reason.")
			return
		end
		
	    self.wndSettings:Show(false, true)
		
		-- if the xmlDoc is no longer needed, you should set it to nil
		-- self.xmlDoc = nil
		
		-- Register handlers for events, slash commands and timer, etc.
		-- e.g. Apollo.RegisterEventHandler("KeyDown", "OnKeyDown", self)
		Apollo.RegisterSlashCommand("bbb", "OnBetterBotBarOn", self)
		Apollo.RegisterSlashCommand("BBB", "OnBetterBotBarOn", self)

		-- Do additional Addon initialization here
		self.onXmlDocLoadedCalled = true
		self:UpdateSettings()
	end
end

-----------------------------------------------------------------------------------------------
-- BetterBotBar Functions
-----------------------------------------------------------------------------------------------
-- Define general functions here
function BetterBotBar:OnConfigure()
	self.wndSettings:Invoke()
end

function BetterBotBar:OnShowActionBarShortcut(nWhichBar, bIsVisible, nNumShortcuts)

	if nWhichBar ~= 1 or not self.wndMain or not self.wndMain:IsValid() then -- 1 is hardcoded to be the engineer pet bar
		return
	end
							
	if userSettings.bNeverShowBotBar then
		self.wndMain:FindChild("PetBtn"):Show(false) 
		self.wndMain:FindChild("PetBarContainer"):Show(false) 
	else
		self.wndMain:FindChild("PetBtn"):Show(bIsVisible) 
		self.wndMain:FindChild("PetBarContainer"):Show(bIsVisible) 
	end
	
	if userSettings.bPetBtnPressed then
		self.wndMain:FindChild("PetBarContainer"):Show(false)
		self.wndMain:FindChild("PetBtn"):SetCheck(true)
	end
	
	if isFirstBotCall then
		self.wndMain:FindChild("PetBarContainer"):SetAnchorOffsets(userSettings.barPosition[1], userSettings.barPosition[2], userSettings.barPosition[3], userSettings.barPosition[4])	
		
		if userSettings.bLockBar then
			self.wndMain:FindChild("PetBarContainer"):SetStyle("Moveable", not userSettings.bLockBar)
		end
			
		isFirstBotCall = false
	else
		userSettings.barPosition[1], userSettings.barPosition[2], userSettings.barPosition[3], userSettings.barPosition[4] = self.wndMain:FindChild("PetBarContainer"):GetAnchorOffsets()
		self.wndMain:FindChild("PetBarContainer"):SetAnchorOffsets(userSettings.barPosition[1], userSettings.barPosition[2], userSettings.barPosition[3], userSettings.barPosition[4])
	end

end

function BetterBotBar:OnPetBtn(wndHandler, wndControl)
	local wndPetContainer = self.wndMain:FindChild("PetBarContainer")
	local bIsShown = not wndPetContainer:IsShown()
	wndPetContainer:Show(bIsShown)
	userSettings.bPetBtnPressed = not bIsShown
	
end

function BetterBotBar:SaveBarPosition(tPositions)
	tPositions[1], tPositions[2], tPositions[3], tPositions[4] = crbClassResources.wndMain:FindChild("PetBarContainer"):GetAnchorOffsets()
end

function BetterBotBar:SetBarPosition(tPositions)
	crbClassResources.wndMain:FindChild("PetBarContainer"):SetAnchorOffsets(tPositions[1], tPositions[2], tPositions[3], tPositions[4])
end

function BetterBotBar:SetBarVisible(bVisible)
	crbClassResources.wndMain:FindChild("PetBarContainer"):Show(bVisible, true)
	crbClassResources.wndMain:FindChild("PetBtn"):Show(bVisible, true)
end

function BetterBotBar:SetLockBar(bLock)
	crbClassResources.wndMain:FindChild("PetBarContainer"):SetStyle("Moveable", not bLock)
end

-- on SlashCommand "/bbb"
function BetterBotBar:OnBetterBotBarOn()
	self.wndSettings:Invoke() -- show the window
end


---------------------------------------------------------------------------------------------------
-- BbbSettings Functions
---------------------------------------------------------------------------------------------------
function BetterBotBar:OnClose()
	self.wndSettings:Close() -- hide the window
end

function BetterBotBar:OnDefault()
	self:SetBarPosition(defaultSettings.barPosition)
	self:SaveBarPosition(userSettings.barPosition)
end

function BetterBotBar:OnCheck( wndHandler, wndControl, eMouseButton )
	self:SetBarVisible(false)
	userSettings.bNeverShowBotBar = true
end

function BetterBotBar:OnUncheck( wndHandler, wndControl, eMouseButton )
	self:SetBarVisible(true)
	userSettings.bNeverShowBotBar = false
end

function BetterBotBar:LockOnCheck( wndHandler, wndControl, eMouseButton )
	self:SetLockBar(true)
	userSettings.bLockBar = true
end

function BetterBotBar:LockOnUncheck( wndHandler, wndControl, eMouseButton )
	self:SetLockBar(false)
	userSettings.bLockBar = false
end

function BetterBotBar:UpdateSettings()
	if not self.onRestoreCalled or not self.onXmlDocLoadedCalled then
		return
	end
	
	self.wndSettings:FindChild("NeverShowBotBarButton"):SetCheck(userSettings.bNeverShowBotBar)
	self.wndSettings:FindChild("LockBarBtn"):SetCheck(userSettings.bLockBar)
	
end
-----------------------------------------------------------------------------------------------
-- BetterBotBar Instance
-----------------------------------------------------------------------------------------------
local BetterBotBarInst = BetterBotBar:new()
BetterBotBarInst:Init()
